"""
Copyright (c) 2017 James Patrick Dill

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import sys
import inspect

PY3 = sys.version_info[0] == 3
PY2 = sys.version_info[0] == 2


def isfunc(o):
    return inspect.ismethod(o) or inspect.isfunction(o)


if PY3 and sys.version_info[1] >= 3:  # textwrap.indent added in 3.3
    from textwrap import indent as _indent


    def indent(text, amount, ch=" "):
        return _indent(text, amount * ch)

else:
    def indent(text, amount, ch=' '):
        padding = amount * ch
        return "".join(padding + line for line in text.splitlines(True))


def cast_type(obj, *types):
    """
    Attempts to cast object to type. If first type fails, goes through args attempting all given types.

    :param obj: Object to cast to type
    :param types: Types to attempt
    :returns: Object in new type, if converted. Otherwise raises TypeError
    """
    if type(obj) in types:
        return obj

    for t in types:
        try:
            return t(obj)
        except Exception:
            pass

    raise TypeError("Failed to cast {!r} to {}".format(obj, ", ".join([repr(t) for t in types])))
